@@ -1,9 +1,6 @@
 //! Main cache implementation and traits.
 
-use std::{ 
-    path::Path,
-    collections::HashMap,
-};
+use std::path::Path;
 
 use nom::{
     combinator::cond,
@@ -15,8 +12,8 @@ use nom::{
 use crate::{ 
     store::Store, 
     cksm::{ Checksum, Entry },
-    idx::Index,
-    arc::{ self, Archive },
+    idx::Indices,
+    arc::{ Archive, ArchiveRef },
     error::ReadError, 
     util,
     codec,
@@ -27,7 +24,6 @@ use whirlpool::{ Whirlpool, Digest };
 
 pub const MAIN_DATA: &str = "main_file_cache.dat2";
 pub const MAIN_MUSIC_DATA: &str = "main_file_cache.dat2m";
-pub const IDX_PREFIX: &str = "main_file_cache.idx";
 pub const REFERENCE_TABLE: u8 = 255;
 
 /// The core of a cache.
@@ -39,15 +35,16 @@ pub trait CacheCore: CacheRead + Sized {
 pub trait CacheRead {
     fn read(&self, index_id: u8, archive_id: u32) -> crate::Result<Vec<u8>>;
     #[inline]
-    fn read_archive(&self, archive: &Archive) -> crate::Result<Vec<u8>> {
+    fn read_archive(&self, archive: &ArchiveRef) -> crate::Result<Vec<u8>> {
         self.read(archive.index_id, archive.id)
     }
 }
 
 /// Main cache struct providing basic utilities.
+#[derive(Clone, Debug)]
 pub struct Cache<S: Store> {
     store: S,
-	pub indices: HashMap<u8, Index>
+	indices: Indices,
 }
 
 impl<S: Store> Cache<S> {
@@ -105,7 +102,7 @@ impl<S: Store> Cache<S> {
     }
 
     #[inline]
-    pub fn read_archive(&self, archive: &Archive) -> crate::Result<Vec<u8>> {
+    pub fn read_archive(&self, archive: &ArchiveRef) -> crate::Result<Vec<u8>> {
         CacheRead::read_archive(self, archive)
     }
 
@@ -113,7 +110,7 @@ impl<S: Store> Cache<S> {
     /// that the client received during the update protocol.
     /// 
     /// NOTE: The RuneScape client doesn't have a valid crc for index 16.
-    /// This checksum sets the crc and revision for index 16 both to 0.
+    /// This checksum sets the crc and version for index 16 both to 0.
     /// The crc for index 16 should be skipped.
     /// 
     /// # Errors
@@ -142,7 +139,7 @@ impl<S: Store> Cache<S> {
                     let data = codec::decode(&buffer)?;
                     
                     let (_, version) = cond(data[0] >= 6, be_u32)(&data[1..5])?;
-                    let version = if let Some(version) = version { version } else { 0 };
+                    let version = version.unwrap_or(0);
 
                     let mut hasher = Whirlpool::new();
                     hasher.update(&buffer);
@@ -151,7 +148,7 @@ impl<S: Store> Cache<S> {
                     checksum.push(
                         Entry { 
                             crc: crc32::checksum_ieee(&buffer), 
-                            revision: version,
+                            version,
                             hash
                         }
                     );
@@ -218,28 +215,24 @@ impl<S: Store> Cache<S> {
     /// # }
     /// ```
     #[inline]
-    pub fn archive_by_name<T: Into<String>>(&self, index_id: u8, name: T) -> crate::Result<Archive> {
+    pub fn archive_by_name<T: Into<String>>(&self, index_id: u8, name: T) -> crate::Result<ArchiveRef> {
         let name = name.into();
 
-        let index = match self.indices.get(&index_id) {
-            Some(index) => index,
-            None => return Err(ReadError::IndexNotFound(index_id).into())
-        };
+        let index = self.indices.get(&index_id)
+            .ok_or(ReadError::IndexNotFound(index_id))?;
         let hash = util::djd2::hash(&name);
 
         let buffer = self.read(REFERENCE_TABLE, index_id as u32)?;
         let data = codec::decode(&buffer)?;
 
-        let archives = arc::parse_archive_data(&data)?;
+        let archives = Archive::parse(&data)?;
 
-        for archive_data in archives {
-            if archive_data.name_hash == hash {
-                match index.archives.get(&(archive_data.id as u32)) {
-                    Some(archive) => return Ok(*archive),
-                    None => return Err(
-                        ReadError::ArchiveNotFound(index_id, archive_data.id as u32).into()
-                    ),
-                }
+        for archive in archives {
+            if archive.name_hash == hash {
+                let archive = index.archives.get(&(archive.id as u32))
+                    .ok_or(ReadError::ArchiveNotFound(index_id, archive.id as u32))?;
+
+                return Ok(*archive)
             }
         }
 
@@ -267,6 +260,11 @@ impl<S: Store> Cache<S> {
     pub fn index_count(&self) -> usize {
         self.indices.len()
     }
+
+    #[inline]
+    pub fn indices(&self) -> &Indices {
+        &self.indices
+    }
 }
 
 impl<S: Store> CacheCore for Cache<S> {
@@ -275,7 +273,7 @@ impl<S: Store> CacheCore for Cache<S> {
         let path = path.as_ref();
 
         let store = util::load_store(path)?;
-        let indices = util::load_indices(path)?;
+        let indices = Indices::new(path)?;
 
         Ok(Self { store, indices })
     }
@@ -284,15 +282,11 @@ impl<S: Store> CacheCore for Cache<S> {
 impl<S: Store> CacheRead for Cache<S> {
     #[inline]
     fn read(&self, index_id: u8, archive_id: u32) -> crate::Result<Vec<u8>> {
-        let index = match self.indices.get(&index_id) {
-            Some(index) => index,
-            None => return Err(ReadError::IndexNotFound(index_id).into())
-        };
+        let index = self.indices.get(&index_id)
+            .ok_or(ReadError::IndexNotFound(index_id))?;
 
-        let archive = match index.archives.get(&archive_id) {
-            Some(archive) => archive,
-            None => return Err(ReadError::ArchiveNotFound(index_id, archive_id).into())
-        };
+        let archive = index.archives.get(&archive_id)
+            .ok_or(ReadError::ArchiveNotFound(index_id, archive_id))?;
 
         self.store.read(archive)
     }
