@@ -17,43 +17,11 @@ use crate::{
     Store,
     Cache,
     Definition,
-    arc,
+    arc::{ Archive, ArchiveFileGroup },
 };
 
-// #[macro_use]
-// macro_rules! impl_osrs_loader {
-//    ($ldr:ident, $def:ty, $defs_field:ident, $parse_func:ident, $($arg: expr),*) => {
-//         impl $ldr {
-//             #[inline]
-//             pub fn new<S: Store>(cache: &Cache<S>) -> crate::Result<Self> {
-//                 Loader::new(cache)
-//             }
-
-//             #[inline]
-//             pub fn load(&self, id: u16) -> Option<&$def> {
-//                 Loader::load(self, id)
-//             }
-//         }
-
-//         impl Loader<$def> for $ldr {
-//             #[inline]
-//             fn new<S: Store>(cache: &Cache<S>) -> crate::Result<$ldr> {
-//                 // let $defs_field = util::osrs::parse_defs_from_archive(cache, $idx_id, $arc_id)?;
-//                 let $defs_field = $parse_func(cache, $($arg),*)?;
-
-//                 Ok($ldr { $defs_field })
-//             }
-
-//             #[inline]
-//             fn load(&self, id: u16) -> Option<&$def> {
-//                 self.$defs_field.get(&id)
-//             }
-//         }
-//    };
-// }
-
 macro_rules! impl_osrs_loader {
-   ($ldr:ident, $def:ty, $defs_field:ident, index_id: $idx_id:expr $(, archive_id: $arc_id:expr)?) => {
+   ($ldr:ident, $def:ty, index_id: $idx_id:expr $(, archive_id: $arc_id:expr)?) => {
         impl $ldr {
             #[inline]
             pub fn new<S: Store>(cache: &Cache<S>) -> crate::Result<Self> {
@@ -73,23 +41,23 @@ macro_rules! impl_osrs_loader {
             #[inline]
             fn new<S: Store>(cache: &Cache<S>) -> crate::Result<Self> {            
                 $(
-                    let $defs_field = crate::util::osrs::parse_defs_from_archive(cache, $idx_id, $arc_id)?;
+                    let map = crate::util::osrs::parse_defs_from_archive(cache, $idx_id, $arc_id)?;
 
-                    return Ok($ldr { $defs_field });
+                    return Ok(Self(map));
                 )?
 
-                let $defs_field = crate::util::osrs::parse_defs(cache, $idx_id)?;
+                let map = crate::util::osrs::parse_defs(cache, $idx_id)?;
 
-                Ok(Self { $defs_field })
+                Ok(Self(map))
             }
 
             #[inline]
             fn load(&self, id: u32) -> Option<&Self::Definition> {
-                self.$defs_field.get(&id)
+                self.0.get(&id)
             }
         }
 
-        impl_iter_for_loader!($ldr, $def, $defs_field);
+        impl_iter_for_loader!($ldr, $def);
    };
 }
 
@@ -131,40 +99,42 @@ pub fn load_map_def<S: Store>(cache: &Cache<S>, region_id: u32) -> crate::Result
 /// # }
 /// ```
 #[inline]
-pub fn parse_defs_from_archive<T: Definition, S: Store>(cache: &Cache<S>, index_id: u8, archive_id: u32) -> crate::Result<HashMap<u32, T>> {
+pub fn parse_defs_from_archive<D: Definition, S: Store>(cache: &Cache<S>, index_id: u8, archive_id: u32) -> crate::Result<HashMap<u32, D>> {
     let buffer = cache.read(REFERENCE_TABLE, index_id as u32)?;
     let buffer = codec::decode(&buffer)?;
     
-    let archives = arc::parse_archive_data(&buffer)?;
+    let archives = Archive::parse(&buffer)?;
     let entry_count = archives[archive_id as usize - 1].entry_count;
     
     let buffer = cache.read(index_id, archive_id)?;
     let buffer = codec::decode(&buffer)?;
 
-    let archive_data = arc::parse_content(&buffer, entry_count)?;
+    let archive_group = ArchiveFileGroup::parse(&buffer, entry_count)?;
 
     let mut definitions = HashMap::new();
-    for (id, buffer) in archive_data {
-        definitions.insert(id, T::new(id, &buffer)?);
+    for archive_file in archive_group {
+        definitions.insert(archive_file.id, D::new(archive_file.id, &archive_file.data)?);
     }
 
     Ok(definitions)
 }
 
+// TODO: document when model loader is implemented
 // every archive is 1 def, not like the one above where one archive contains many defs
+// TODO: remove (crate) when ready
 #[inline]
-pub fn parse_defs<T: Definition, S: Store>(cache: &Cache<S>, index_id: u8) -> crate::Result<HashMap<u32, T>> {
+pub(crate) fn parse_defs<D: Definition, S: Store>(cache: &Cache<S>, index_id: u8) -> crate::Result<HashMap<u32, D>> {
     let buffer = cache.read(REFERENCE_TABLE, index_id as u32)?;
     let buffer = codec::decode(&buffer)?;
     
-    let archives = arc::parse_archive_data(&buffer)?;
+    let archives = Archive::parse(&buffer)?;
     let mut definitions = HashMap::new();
     
     for archive in &archives {
         let buffer = cache.read(index_id, archive.id as u32)?;
         let buffer = codec::decode(&buffer)?;
 
-        definitions.insert(archive.id, T::new(archive.id, &buffer)?);
+        definitions.insert(archive.id, D::new(archive.id, &buffer)?);
     }
 
     Ok(definitions)
